import Automator
import objc
from PyObjCTools.TestSupport import TestCase


class TestAutomator(TestCase):
    def testClasses(self):
        self.assertHasAttr(Automator, "AMAction")
        self.assertIsInstance(Automator.AMAction, objc.objc_class)

        self.assertHasAttr(Automator, "AMAppleScriptAction")
        self.assertIsInstance(Automator.AMAppleScriptAction, objc.objc_class)

    def testInformalProtocols(self):
        self.assertNotHasAttr(Automator, "protocols")


class TestCallableMetadata(TestCase):
    def test_callable_metadata_is_sane(self):
        self.assertCallableMetadataIsSane(
            Automator,
            exclude_attrs={
                (
                    "NSObject",
                    "copyRenderedTextureForCGLContext_pixelFormat_bounds_isFlipped_",
                )
            },
        )



import Automator
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestAMWorkflow(TestCase):
    @min_os_level("10.5")
    def testMethods(self):
        self.assertArgIsOut(Automator.AMWorkflow.runWorkflowAtURL_withInput_error_, 2)
        self.assertArgIsOut(Automator.AMWorkflow.initWithContentsOfURL_error_, 1)
        self.assertResultIsBOOL(Automator.AMWorkflow.writeToURL_error_)
        self.assertArgIsOut(Automator.AMWorkflow.writeToURL_error_, 1)
        self.assertResultIsBOOL(Automator.AMWorkflow.setValue_forVariableWithName_)



import Automator
from PyObjCTools.TestSupport import TestCase


class TestAMShellScriptAction(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Automator.AMShellScriptAction.remapLineEndings)



import Automator
from PyObjCTools.TestSupport import TestCase


class TestAMBundleAction(TestCase):
    def testMethods(self):
        self.assertArgIsBOOL(
            Automator.AMBundleAction.initWithDefinition_fromArchive_, 1
        )
        self.assertResultIsBOOL(Automator.AMBundleAction.hasView)



import objc
from Automator import AMWorkflow
from Cocoa import (
    NSURL,
    NSBeginInformationalAlertSheet,
    NSBundle,
    NSIndexSet,
    NSMutableArray,
    NSObject,
)


class Controller(NSObject):
    # interface builder variables
    myWindow = objc.IBOutlet()
    workflowView = objc.IBOutlet()
    workflowTable = objc.IBOutlet()
    workflowController = objc.IBOutlet()
    tableContent = objc.IBOutlet()

    # instance variables
    _workflows = objc.ivar()
    _runningWorkflow = objc.ivar.bool()

    def workflows(self):
        return self._workflows

    def setWorkflows_(self, value):
        self._workflows = value

    def runningWorkflow(self):
        return self._runningWorkflow

    def setRunningWorkflow_(self, value):
        self._runningWorkflow = value

    # display the selected workflow.
    def displaySelectedWorkflow(self):
        # get the selected row from the workflow table.
        theRow = self.workflowTable.selectedRow()

        # if there is a selection and we are not running
        # the selected workflow, then we can display the
        # workflow selected in the list.
        if theRow != -1 and not self._.runningWorkflow:
            # retrieve the first item in the selection
            selectedEntry = self.tableContent.arrangedObjects()[theRow]

            # retrieve the selected application from our list of applications.
            selectedWorkflow = selectedEntry["workflow"]

            # ask the AMWorkflowController to display the selected workflow
            self.workflowController.setWorkflow_(selectedWorkflow)

            # set the window title
            self.myWindow.setTitle_(selectedEntry["name"])

            return True

        else:
            return False

    # awakeFromNib is called after our MainMenu.nib file has been loaded
    # and the main window is ready for use.  Here, we finish initializing
    # our content for display in the window.
    def awakeFromNib(self):
        # we're only using the AMWorkflowView for display
        self.workflowView.setEditable_(False)

        # set up the data for NSTableView.  We'll store a list of
        # NSDictonary records each containing some information about the
        # workflow.  We'll display the name of the workflow's file in the
        # window.

        # set up an array for storing the table information
        theWorkflows = NSMutableArray.alloc().initWithCapacity_(20)

        # retrieve a list of all of the workflows stored in the application's
        # resourced folder.
        workflowPaths = NSBundle.mainBundle().pathsForResourcesOfType_inDirectory_(
            "workflow", "workflows"
        )

        # iterate through the paths, adding them to our table information
        # as we go.
        for nthWorkflowPath in workflowPaths:
            wfError = None

            # convert the path into an URL
            nthWorkflowURL = NSURL.fileURLWithPath_isDirectory_(nthWorkflowPath, False)

            # allocate and initialize the workflow
            nthWorkflow, wfError = AMWorkflow.alloc().initWithContentsOfURL_error_(
                nthWorkflowURL, None
            )

            if nthWorkflow:
                # calculate the file name without path or extension
                nthFileName = nthWorkflowPath.componentsSeparatedByString_("/")[-1]
                nthDisplayName = nthFileName[:-9]

                # add the workflow to the list
                theWorkflows.append(
                    {
                        "name": nthDisplayName,
                        "path": nthWorkflowPath,
                        "workflow": nthWorkflow,
                    }
                )

        # set the workflows
        self._.workflows = theWorkflows

        # if there are any workflows in the list, then select and display the first one */
        if len(self._.workflows):
            self.workflowTable.selectRowIndexes_byExtendingSelection_(
                NSIndexSet.indexSetWithIndex_(0), False
            )
            self.displaySelectedWorkflow()

    # NSApplication delegate method - for convenience. We have set the
    # File's Owner's delegate to our Controller object in the MainMenu.nib file.
    def applicationShouldTerminateAfterLastWindowClosed_(self, sender):
        return True

    # NSTableView delegate methods.
    # We have set our controller object as the NSTableView's delegate in the MainMenu.nib file.

    # selectionShouldChangeInTableView: is called when the user has clicked in the
    # table view in a way that will cause the selection to change.  This method allows us
    # to decide if we would like to allow the selection to change and the response
    # we return depends on if we are running a selection - we don't allow the selection
    # to change while a workflow is running.
    def selectionShouldChangeInTableView_(self, tableView):
        # if the current selected workflow is running, don't change the selection.
        if self._.runningWorkflow:
            # display an alert explaining why the selection cannot be changed.

            # get the name of the action that is running now.
            selectedWorkflow = self.tableContent.arrangedObjects()[
                self.workflowTable.selectedRow()
            ]["name"]

            # display a modal sheet explaining why the selection cannot be changed.
            NSBeginInformationalAlertSheet(
                f"The '{selectedWorkflow}' action is running.",
                "OK",
                None,
                None,
                self.myWindow,
                None,
                None,
                None,
                None,
                "You cannot select another action until the '%s' action has finished running."
                % (selectedWorkflow),
            )

        # return true only if we are not in the middle of running an action.
        return not self._.runningWorkflow

    # tableViewSelectionIsChanging: is called after the selection has changed.  All there
    # is to do here is update the workflow displayed in the AMWorkflowView to show the newly
    # selected workflow.
    def tableViewSelectionIsChanging_(self, notification):
        self.displaySelectedWorkflow()

    # AMWorkflowController delegate methods.  In these routines we adjust the
    # value of our runningWorkflow property.  Key value bindings to this property
    # defined in our interface file take care of enabling/disabling the run/stop buttons,
    # and displaying the progress bar.
    def workflowControllerWillRun_(self, controller):
        self._.runningWorkflow = True

    def workflowControllerDidRun_(self, controller):
        self._.runningWorkflow = False

    def workflowControllerDidStop_(self, controller):
        self._.runningWorkflow = False



"""
Wrappers for the "Automator" framework on macOS. The Automator framework
supports the development of actions for the Automator application, as well
as the ability to run a workflow in developer applications. An action is
a bundle that, when loaded and run, performs a specific task.

These wrappers don't include documentation, please check Apple's documentation
for information on how to use this framework and PyObjC's documentation
for general tips and tricks regarding the translation between Python
and (Objective-)C frameworks
"""
from pyobjc_setup import setup

VERSION = "9.1b1"

setup(
    name="pyobjc-framework-Automator",
    description="Wrappers for the framework Automator on macOS",
    packages=["Automator"],
    version=VERSION,
    install_requires=["pyobjc-core>=" + VERSION, "pyobjc-framework-Cocoa>=" + VERSION],
    long_description=__doc__,
)



import Automator
from PyObjCTools.TestSupport import TestCase


class TestAutomatorErrors(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(Automator.AMErrorCode)

    def testConstants(self):
        self.assertEqual(Automator.AMAutomatorErrorDomain, "com.apple.Automator")
        self.assertEqual(Automator.AMActionErrorKey, "AMActionErrorKey")
        self.assertIsInstance(Automator.AMAutomatorErrorDomain, str)
        self.assertIsInstance(Automator.AMActionErrorKey, str)

        self.assertEqual(Automator.AMWorkflowNewerVersionError, -100)
        self.assertEqual(Automator.AMWorkflowPropertyListInvalidError, -101)
        self.assertEqual(Automator.AMWorkflowNewerActionVersionError, -111)
        self.assertEqual(Automator.AMWorkflowOlderActionVersionError, -112)
        self.assertEqual(Automator.AMWorkflowActionsNotLoadedError, -113)
        self.assertEqual(Automator.AMWorkflowNoEnabledActionsError, -114)
        self.assertEqual(Automator.AMUserCanceledError, -128)
        self.assertEqual(Automator.AMNoSuchActionError, -200)
        self.assertEqual(Automator.AMActionNotLoadableError, -201)
        self.assertEqual(Automator.AMActionArchitectureMismatchError, -202)
        self.assertEqual(Automator.AMActionRuntimeMismatchError, -203)
        self.assertEqual(Automator.AMActionLoadError, -204)
        self.assertEqual(Automator.AMActionLinkError, -205)
        self.assertEqual(Automator.AMActionApplicationResourceError, -206)
        self.assertEqual(Automator.AMActionApplicationVersionResourceError, -207)
        self.assertEqual(Automator.AMActionFileResourceError, -208)
        self.assertEqual(Automator.AMActionLicenseResourceError, -209)
        self.assertEqual(Automator.AMActionRequiredActionResourceError, -210)
        self.assertEqual(Automator.AMActionInitializationError, -211)
        self.assertEqual(Automator.AMActionExecutionError, -212)
        self.assertEqual(Automator.AMActionExceptionError, -213)
        self.assertEqual(Automator.AMActionPropertyListInvalidError, -214)
        self.assertEqual(Automator.AMActionInsufficientDataError, -215)
        self.assertEqual(Automator.AMActionIsDeprecatedError, -216)
        self.assertEqual(Automator.AMActionFailedGatekeeperError, -217)
        self.assertEqual(Automator.AMActionSignatureCorruptError, -218)
        self.assertEqual(Automator.AMActionQuarantineError, -219)
        self.assertEqual(Automator.AMActionXProtectError, -220)
        self.assertEqual(Automator.AMActionMalwareError, -221)
        self.assertEqual(Automator.AMActionThirdPartyActionsNotAllowedError, -222)
        self.assertEqual(Automator.AMActionXPCError, -223)
        self.assertEqual(Automator.AMConversionNotPossibleError, -300)
        self.assertEqual(Automator.AMConversionNoDataError, -301)
        self.assertEqual(Automator.AMConversionFailedError, -302)



import Automator
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestAMWorkflowView(TestCase):
    @min_os_level("10.5")
    def testMethods(self):
        self.assertResultIsBOOL(Automator.AMWorkflowView.isEditable)
        self.assertArgIsBOOL(Automator.AMWorkflowView.setEditable_, 0)



import Automator
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level


class TestAMWorkflowController(TestCase):
    @min_os_level("10.5")
    def testMethods(self):
        self.assertResultIsBOOL(Automator.AMWorkflowController.canRun)
        self.assertResultIsBOOL(Automator.AMWorkflowController.isRunning)
        self.assertResultIsBOOL(Automator.AMWorkflowController.isPaused)

    @min_sdk_level("10.13")
    def testProtocols(self):
        self.assertProtocolExists("AMWorkflowControllerDelegate")
